/***************************************************************************
                          ksmsarchive.cpp  -  description
                             -------------------
    begin                : Mon Oct 28 2002
    copyright            : (C) 2002 by Michael Schuerig
    email                : michael@schuerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kcursor.h>
#include <klocale.h>
#include <kaction.h>
#include <kapplication.h>
#include <kdebug.h>
#include <kedittoolbar.h>
#include <kkeydialog.h>
#include <kpopupmenu.h>
#include <kstdaction.h>
#include <kstatusbar.h>
#include <ksystemtray.h>
#include <qglobal.h>
#include <qprogressbar.h>
#include <qtimer.h>

#include "actionutils.h"
#include "ksmsapp.h"
#include "ksmsarchivedoc.h"
#include "ksmsarchiveview.h"
#include "ksmsprefs.h"
#include "messagestore.h"
#include "mobilephonebook.h"
#include "msgsender.h"

#include "ksmsarchive.h"


namespace {

class KSmsSystemTray : public KSystemTray {
public:
    KSmsSystemTray( KAction* newMessageAction,
                    KAction* configureAction,
                    QWidget* parent = 0, const char* name = 0 )
        : KSystemTray( parent, name ),
          newMessageAction_(newMessageAction)
    {
        newMessageAction->plug( contextMenu() );
        configureAction->plug( contextMenu() );
    }
protected:
    void mousePressEvent( QMouseEvent* me )
    {
        if ( me->button() == MidButton ) {
            newMessageAction_->activate();
        }
        else {
            KSystemTray::mousePressEvent( me );
        }
    }
private:
    KAction* newMessageAction_;
};

}


const QString KSmsArchive::UI_RESOURCE_FILE( "ksmsarchiveui.rc" );
const QString KSmsArchive::WINDOW_NAME( "ArchiveWindow" );


KSmsArchive::KSmsArchive()
    : KMainWindow(0),
      document_(0), view_(0), newMsgAction_(0),
      toolbarAction_(0), statusbarAction_(0)
{
    setWFlags( Qt::WDestructiveClose );
    resize( 600, 400 );

    initDocument();
    initStatusBar();
    initView();
    initActions();
    initTray();
    setAutoSaveSettings( WINDOW_NAME );

    toolbarAction_->setChecked( !toolBar()->isHidden() );
    statusbarAction_->setChecked( !statusBar()->isHidden() );

    //### move this to some better place
    connect( application(), SIGNAL(configChanged()),
             this, SLOT(configChanged()) );

    //### Sigh. Move this to some better place
    view_->setMessagePopup(
        dynamic_cast<QPopupMenu*>(factory()->container("message_context_popup", this)) );
             
    QTimer::singleShot( 0, this, SLOT(readMessages()) );
}


KSmsArchive::~KSmsArchive()
{
}


void
KSmsArchive::initDocument()
{
    document_ = new KSmsArchiveDoc( application()->phoneBook(),
                                    application()->messageStore(),
                                    this, "Archive Doc" );
    connect( document_, SIGNAL(progress(unsigned long, unsigned long)),
             this, SLOT(showProgress(unsigned long)) );
}


void
KSmsArchive::initView()
{
    Q_ASSERT( document_ != 0 );

    view_ = new KSmsArchiveView( document_, this, "Archive View" );
    setCentralWidget(view_);
    
    connect( view_, SIGNAL(senderSelected(const MsgSender*)),
             this, SLOT(senderSelected(const MsgSender*)) );
    connect( application(), SIGNAL(configChanged()),
             view_, SLOT(configChanged()) );
}


void
KSmsArchive::initTray()
{
    Q_ASSERT( newMsgAction_ != 0 );
    Q_ASSERT( configureAction_ != 0 );
    
    tray_ = new KSmsSystemTray( newMsgAction_,
                                configureAction_,
                                this );
    tray_->setPixmap( application()->miniIcon() );
    updateTray();
}


void
KSmsArchive::initStatusBar()
{
    Q_ASSERT( document_ != 0 );
    statusBar()->insertItem( i18n(" Ready."), ID_STATUS_GENERAL, 1 );
    statusBar()->setItemAlignment( ID_STATUS_GENERAL, AlignLeft | AlignVCenter );
    progressBar_ = new QProgressBar( 100, statusBar() );
    progressBar_->setMinimumWidth( 100 );
    statusBar()->addWidget( progressBar_, 0, true );

    actionCollection()->setHighlightingEnabled( true );
    connect( actionCollection(), SIGNAL(actionStatusText(const QString&)),
             this, SLOT(statusMessage(const QString&)) );
    connect( actionCollection(), SIGNAL(clearStatusText()),
             this, SLOT(statusMessage()) );
}


void
KSmsArchive::initActions()
{
    Q_ASSERT( document_ != 0 );
    Q_ASSERT( view_ != 0 );

    KAction* action;
    
    newMsgAction_ = new KAction( i18n("&New Message"), "filenew", CTRL + Key_N,
                                 application(), SLOT(newMessage()), actionCollection(),
                                 "new_message" );
    Q_CHECK_PTR( newMsgAction_ );
    newMsgAction_->setToolTip( i18n( "Create a new message.") );

    action = new KAction( i18n("Address Book..."), "contents", 0,
                          application(), SLOT(openAddressBook()), actionCollection(),
                          "open_addressbook" );
    Q_CHECK_PTR( action );
    action->setToolTip( i18n("Open the address book.") );
    
    importFromPhoneAction_ = new KAction( i18n("Import Messages from Phone"), "fileopen", 0,
                                          this, SLOT(importMessagesFromPhone()), actionCollection(),
                                          "import_messages" );
    Q_CHECK_PTR( importFromPhoneAction_ );
    importFromPhoneAction_->setToolTip( i18n("Import messages from a mobile phone.") );

    KStdAction::quit( this, SLOT(quit()), actionCollection() );

    action = KStdAction::copy( view_, SLOT(editCopy()),
                               actionCollection() );
    connect( view_, SIGNAL(copyAvailable(bool)), action, SLOT(setEnabled(bool)) );

    deleteMsgAction_ = new KAction( i18n("Delete Message"), "editdelete", SHIFT+Key_Delete,
                                    view_, SLOT(deleteSelectedMessage()),
                                    actionCollection(),
                                    "delete_message" );
    Q_CHECK_PTR( deleteMsgAction_ );
    deleteMsgAction_->setEnabled( false );
    connect( view_, SIGNAL(deleteMessageAvailable(bool)),
             deleteMsgAction_, SLOT(setEnabled(bool)) );
    deleteMsgAction_->setToolTip( i18n("Delete the selected message.") );
                                                    
    editSenderAction_ = new KAction( i18n("Edit Sender..."), "edit", 0,
                                     this, SLOT(editSender()), actionCollection(),
                                     "edit_sender" );
    Q_CHECK_PTR( editSenderAction_ );
    editSenderAction_->setEnabled( false );
    editSenderAction_->setToolTip( i18n("Open the selected message sender in the address book.") );

    toolbarAction_ = KStdAction::showToolbar( this, SLOT(updateToolbar()), actionCollection() );
    Q_CHECK_PTR( toolbarAction_ );
    
    statusbarAction_ = KStdAction::showStatusbar( this, SLOT(updateStatusBar()), actionCollection() );
    Q_CHECK_PTR( statusbarAction_ );
    
    KStdAction::keyBindings( this, SLOT(configureKeyBindings()), actionCollection() );
    KStdAction::configureToolbars( this, SLOT(configureToolbars()), actionCollection() );
    configureAction_ = KStdAction::preferences( application(), SLOT(configurePreferences()),
                                                actionCollection() );

    createGUI( UI_RESOURCE_FILE );
}


KSmsApp*
KSmsArchive::application() const
{
    return dynamic_cast<KSmsApp*>(KApplication::kApplication());
}


void
KSmsArchive::editSender()
{
    application()->openAddressBook( senderUid_ );
}


void
KSmsArchive::readMessages()
{
    kdDebug() << "KSmsArchive::readMessages()" << endl;

    QApplication::setOverrideCursor( KCursor::waitCursor() );
    
    statusMessage( i18n(" Reading messages...") );
    progressBar_->reset();

    ActionUtils::enableCollection( actionCollection(), false );

    uint oldCount = document_->messageCount();

    document_->readMessages();

    uint newCount = document_->messageCount();

    statusMessage( i18n(" Read %1 messages.").arg(newCount - oldCount) );
    progressBar_->reset();

    QApplication::restoreOverrideCursor();

    ActionUtils::enableCollection( actionCollection(), true );
}


void
KSmsArchive::importMessagesFromPhone()
{
    kdDebug() << "KSmsArchive::importMessagesFromPhone()" << endl;

    QApplication::setOverrideCursor( KCursor::waitCursor() );

    statusMessage( i18n(" Importing messages...") );
    progressBar_->reset();
    
    ActionUtils::enableCollection( actionCollection(), false );

    uint oldCount = document_->messageCount();

    document_->importMessagesFromPhone();

    uint newCount = document_->messageCount();

    statusMessage( i18n(" Imported %1 new messages.").arg(newCount - oldCount) );
    progressBar_->reset();

    QApplication::restoreOverrideCursor();

    ActionUtils::enableCollection( actionCollection(), true );
}


void
KSmsArchive::showProgress( unsigned long current )
{
//    kdDebug() << QString::number(current) << endl;

    progressBar_->setProgress( current );

}


/*
void
KSmsArchive::closeEvent( QCloseEvent* ce )
{
    if ( KSmsPrefs::instance()->ui_dockable ) {
        hide();
        ce->ignore();
    }
    else {
        ce->accept();
    }
}
*/


void
KSmsArchive::quit()
{
    kdDebug() << "KSmsArchive::quit" << endl;

    KApplication::kApplication()->quit();
}


void
KSmsArchive::senderSelected( const MsgSender* sender )
{
    statusMessage( i18n(" %1 received messages. %2 sent messages.")
                        .arg(sender->receivedMessageCount())
                        .arg(sender->sentMessageCount()) );

    updateSenderAction( sender );
}


bool
KSmsArchive::queryClose()
{
    kdDebug() << "KSmsArchive::queryClose" << endl;

    return true;
}


bool
KSmsArchive::queryExit()
{
    kdDebug() << "KSmsArchive::queryExit" << endl;

    saveMainWindowSettings( KGlobal::config(), WINDOW_NAME );
    return true;
}


void
KSmsArchive::configChanged()
{
    updateTray();
}


void
KSmsArchive::updateToolbar()
{
    if ( toolbarAction_->isChecked() ) {
        toolBar()->show();
    }
    else {
        toolBar()->hide();
    }
}


void
KSmsArchive::updateStatusBar()
{
    if ( statusbarAction_->isChecked() ) {
        statusBar()->show();
    }
    else {
        statusBar()->hide();
    }
}


void
KSmsArchive::updateTray()
{
    if ( KSmsPrefs::instance()->ui_dockable ) {
        tray_->show();
    }
    else {
        show();
        tray_->hide();
    }
}


void
KSmsArchive::configureToolbars()
{
    kdDebug() << "KSmsArchive::configureToolbars" << endl;

    saveMainWindowSettings( KGlobal::config(), WINDOW_NAME );
    KEditToolbar dlg( actionCollection(), UI_RESOURCE_FILE );
    connect( &dlg, SIGNAL(newToolbarConfig()), this, SLOT(newToolbarConfig()) );
    if ( dlg.exec() ) {
        createGUI( UI_RESOURCE_FILE );
    }
}


void
KSmsArchive::newToolbarConfig()
{
    applyMainWindowSettings( KGlobal::config(), WINDOW_NAME );
}


void
KSmsArchive::configureKeyBindings()
{
    kdDebug() << "configureKeyBindings" << endl;

    KKeyDialog::configure(actionCollection(), xmlFile());
}


void
KSmsArchive::statusMessage( const QString& msg )
{
    statusBar()->changeItem( msg, ID_STATUS_GENERAL );
}


void
KSmsArchive::updateSenderAction( const MsgSender* sender )
{
    if ( !sender->uid().isEmpty() ) {
        editSenderAction_->setText(
            i18n("Edit %1...").arg( !sender->name().isEmpty()
                                    ? sender->name()
                                   : sender->phoneNumber() ) );
        editSenderAction_->setEnabled( true );
        senderUid_ = sender->uid();
    }
    else {
        editSenderAction_->setText( i18n("Edit Sender...") );
        editSenderAction_->setEnabled( false );
        senderUid_ = QString::null;
    }
}

