/***************************************************************************
                          ksmsarchiveview.cpp  -  description
                             -------------------
    begin                : Mon Oct 28 2002
    copyright            : (C) 2002 by Michael Schuerig
    email                : michael@schuerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kapplication.h>
#include <kdebug.h>
#include <kdialog.h>
#include <kglobal.h>
#include <klocale.h>
#include <kmainwindow.h>
#include <qclipboard.h>
#include <qdatetime.h>
#include <qglobal.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qpopupmenu.h>

#include "ksmsarchivedoc.h"
#include "ksmsprefs.h"
#include "messageformatter.h"
#include "messagelistbox.h"
#include "senderlistbox.h"
#include "ksmsarchiveview.h"



KSmsArchiveView::KSmsArchiveView( KSmsArchiveDoc* document,
                                  QWidget* parent, const char* name )
    : QWidget(parent, name),
      document_(document),
      msgPopup_(0)
{
    Q_ASSERT( document_ != 0 );
    
    initWidgets();
    loadSendersAndMessages();
}


KSmsArchiveView::~KSmsArchiveView()
{
}


void
KSmsArchiveView::initWidgets()
{
    QGridLayout* layout = new QGridLayout( this, 2, 3,
                                           KDialog::marginHint(), KDialog::spacingHint() );

    layout->addWidget( new QLabel(i18n("Sender"), this), 1, 0 );
    sendersList_ = new SenderListBox( KSmsPrefs::instance()->ui_showPhoneNumber, this );
    Q_CHECK_PTR( sendersList_ );
    connect( document_, SIGNAL(sendersChanged()),
             this, SLOT(loadSendersAndMessages()) );
    connect( document_, SIGNAL(messagesChangedForSender(const MsgSender*)),
             this, SLOT(updateSender(const MsgSender*)) );
    connect( sendersList_, SIGNAL(senderSelected(const MsgSender*)),
             this, SLOT(slotSenderSelected(const MsgSender*)) );
    layout->addWidget( sendersList_, 2, 0 );
    layout->setColStretch( 0, 1 );

    layout->addWidget( new QLabel(i18n("Received Message"), this), 1, 1 );
    receivedMessagesList_ = new MessageListBox( KSmsPrefs::instance()->ui_receivedMessageTemplate,
                                                this, "Received Messages List" );
    Q_CHECK_PTR( receivedMessagesList_ );
    connect( receivedMessagesList_, SIGNAL(contextMenuRequested(QListBoxItem*, const QPoint&)),
             this, SLOT(showMsgContextMenu(QListBoxItem*, const QPoint&)) );
    layout->addWidget( receivedMessagesList_, 2, 1 );
    layout->setColStretch( 1, 2 );

    layout->addWidget( new QLabel(i18n("Sent Message"), this), 1, 2 );
    sentMessagesList_ = new MessageListBox( KSmsPrefs::instance()->ui_sentMessageTemplate,
                                            this, "Sent Messages List" );
    Q_CHECK_PTR( sentMessagesList_ );
    connect( sentMessagesList_, SIGNAL(contextMenuRequested(QListBoxItem*, const QPoint&)),
             this, SLOT(showMsgContextMenu(QListBoxItem*, const QPoint&)) );
    layout->addWidget( sentMessagesList_, 2, 2 );
    layout->setColStretch( 2, 2 );

    connect( receivedMessagesList_, SIGNAL(messageSelected(const Message*)),
             sentMessagesList_, SLOT(clearSelection()) );
    connect( sentMessagesList_, SIGNAL(messageSelected(const Message*)),
             receivedMessagesList_, SLOT(clearSelection()) );

    connect( sendersList_, SIGNAL(senderSelected(const MsgSender*)),
             this, SLOT(emitAvailable()) );
    connect( receivedMessagesList_, SIGNAL(messageSelected(const Message*)),
             this, SLOT(emitAvailable()) );
    connect( sentMessagesList_, SIGNAL(messageSelected(const Message*)),
             this, SLOT(emitAvailable()) );
}


void
KSmsArchiveView::setMessagePopup( QPopupMenu* popup )
{
    msgPopup_ = popup;
}


void
KSmsArchiveView::loadSendersAndMessages()
{
    kdDebug() << "KSmsArchiveView::loadSendersAndMessages" << endl;
    
    receivedMessagesList_->clear();
    sentMessagesList_->clear();
    sendersList_->setSenders( document_->senders() );
}


void
KSmsArchiveView::updateSender( const MsgSender* sender )
{
    const MsgSender* selectedSender = sendersList_->selectedSender();
    if ( selectedSender == 0 )
        return;

    if ( selectedSender == sender ) {
        showMessagesFor( sender );
    }
}


void
KSmsArchiveView::configChanged()
{
    kdDebug() << "KSmsArchiveView::configChanged" << endl;
    
    sendersList_->setPhoneNumberVisible( KSmsPrefs::instance()->ui_showPhoneNumber );
    sendersList_->setSenders( document_->senders() );
}


const Message*
KSmsArchiveView::selectedMessage() const
{
    const Message* msg = 0;
    
    if ( receivedMessagesList_->hasFocus() ) {
        msg = receivedMessagesList_->selectedMessage();
    }
    else if ( sentMessagesList_->hasFocus() ) {
        msg = sentMessagesList_->selectedMessage();
    }

    return msg;
}


void
KSmsArchiveView::slotSenderSelected( const MsgSender* sender )
{
    showMessagesFor( sender );    
    emit senderSelected( sender );
}


void
KSmsArchiveView::showMsgContextMenu( QListBoxItem* ,
                                     const QPoint& pos )
{
    kdDebug() << "KSmsArchiveView::showMsgContextMenu" << endl;

    if ( msgPopup_ )
        msgPopup_->exec( pos );
}


void
KSmsArchiveView::editCopy()
{
    const Message* msg = selectedMessage();

    if ( msg != 0 ) {
        MessageFormatter formatter( KSmsPrefs::instance()->ui_clipMessageTemplate, "\n" );
        KApplication::clipboard()->setText( formatter.format(msg) );
    }
}


void
KSmsArchiveView::deleteSelectedMessage()
{
    const Message* msg = selectedMessage();

    if ( msg != 0 ) {
        document_->deleteMessage( msg );
    }
}


void
KSmsArchiveView::showMessagesFor( const MsgSender* sender )
{
    receivedMessagesList_->setMessages( sender->receivedMessages() );
    sentMessagesList_->setMessages( sender->sentMessages() );
}


void
KSmsArchiveView::emitAvailable()
{
    bool avail = selectedMessage() != 0;
    emit copyAvailable( avail );
    emit deleteMessageAvailable( avail );
}


