/***************************************************************************
                          messagestore.cpp  -  description
                             -------------------
    begin                : Tue Oct 29 2002
    copyright            : (C) 2002 by Michael Schuerig
    email                : michael@schuerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kapplication.h>
#include <kdebug.h>
#include <klocale.h>
#include <ksavefile.h>
#include <qdir.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qdatetime.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qtextstream.h>
#include <qxml.h>

#include "phoneutils.h"
#include "progresscallback.h"
#include "threadedphoneaccess.h"
#include "xmlinputsourcewithprogress.h"
#include "messagestore.h"


namespace {

const QString KSMS_DOCTYPE( "ksms_messages" );
const QString KSMS_DOC_VERSION( "1" );


class MessagesParser : public QXmlDefaultHandler {
public:
    MessagesParser( MessageStore* store );
    bool notationDecl( const QString& name, const QString&, const QString& );
    bool startElement( const QString&, const QString&, const QString& name,
                       const QXmlAttributes& attrs);
    bool endElement( const QString&, const QString&, const QString& name );
    bool characters( const QString& text );
    QString errorString();

private:
    enum State { st_outside, st_in_messages, st_in_message };
    MessageStore* store_;
    State state_;
    QString msgType_;
    QString msgTimestamp_;
    QString msgSender_;
    QString msgText_;
    QString error_;
};


QTextStream& operator<<( QTextStream& os, const Message& msg );

}


struct MessageStore::ReaderProgress : public ProgressCallback {
    ReaderProgress( MessageStore* store_ )
        : store(store_) {}
    ~ReaderProgress() {}

    void progress( unsigned long current, unsigned long max )
    {
        kapp->processEvents(200);
        store->progress( current, max );
    }

    MessageStore* store;
};



MessageStore::MessageStore( KSharedPtr<MsgSender::Factory> senderFactory,
                            const QString& messagesFile )
    : modified_(false),
      senderFactory_(senderFactory),
      messagesFile_(messagesFile)
{
    Q_ASSERT( senderFactory_ != 0 );

    initMyself();

    connect( ThreadedPhoneAccess::instance(),
             SIGNAL(retrievedMessage(MessageConstants::Type, const QString&,
                                     const QDateTime&, const QString&)),
             this,
             SLOT(addMessage(MessageConstants::Type, const QString&,
                             const QDateTime&, const QString&)) );
}


MessageStore::~MessageStore()
{
}


void
MessageStore::initMyself()
{
    myself_ = senderFactory_->newMyselfSender( senders_ );
}


uint
MessageStore::messageCount() const
{
    return messages_.count();
}


const MessageList
MessageStore::messages() const
{
    return messages_;
}


const MsgSenderList
MessageStore::senders() const
{
    return senders_;
}


void
MessageStore::readMessages( const QString& messagesFile )
{
    switchMessagesFile( messagesFile );

    kdDebug() << "MessageStore::readMessages from " << messagesFile_ << endl;

    Q_ASSERT( !messagesFile_.isEmpty() );

    reading_ = true;
    
    messages_.clear();

    QFile xmlFile( messagesFile_ );
    MessageStore::ReaderProgress progress( this );
    XmlInputSourceWithProgress source( &xmlFile, &progress );
    QXmlSimpleReader reader;
    MessagesParser handler( this );
    reader.setContentHandler( &handler );
    reader.setDTDHandler( &handler );
    source.reset();
    reader.parse( source );

    reading_ = false;

    emit storeChanged();
    
    modified_ = false;
}


void
MessageStore::importMessagesFromPhone()
{
    kdDebug() << "MessageStore::importMessagesFromPhone" << endl;

    ThreadedPhoneAccess::instance()->retrieveMessages();

    kdDebug() << "    Messages read" << endl;
}


void
MessageStore::addMessage( MessageConstants::Type type,
                          const QString& phoneNumber,
                          const QDateTime& timestamp,
                          const QString& text,
                          bool onlyIfNew )
{
    kdDebug() << "ADD MESSAGE" << endl; //####

    if (    type != MessageConstants::SMS_DELIVER
         && type != MessageConstants::SMS_SUBMIT ) {
        kdWarning() << "Message type not handled. It's neither deliver nor submit." << endl;
        return;
    }

    const QString normNumber( PhoneUtils::normalizePhoneNumber(phoneNumber) );
    
    if ( onlyIfNew ) {
        if ( messages_.find(normNumber, timestamp, text) != 0 ) {
            kdDebug() << "Duplicate message not added." << endl;
            return;
        }
    }

    MsgSender* other = senders_.findByPhoneNumber( normNumber );
    if ( other == 0 ) {
        other = newSender( normNumber );
    }

    const Message* msg;

    switch ( type ) {
    case MessageConstants::SMS_DELIVER:
        msg = messages_.newMessage( type, normNumber, other, myself_,
                                    timestamp, text );
        other->addSentMessage( msg );
        myself_->addReceivedMessage( msg );
        break;
    case MessageConstants::SMS_SUBMIT:
        msg = messages_.newMessage( type, normNumber, myself_, other,
                                    timestamp, text );
        myself_->addSentMessage( msg );
        other->addReceivedMessage( msg );
        break;
    default:
        break;
    }
    
    if ( !reading_ ) {
        emit messagesChangedForSender( myself_ );
        emit messagesChangedForSender( other );
    }
    
    modified_ = true;
}


void
MessageStore::addSentMessage( const QString& receiverNumber,
                              const QString& text )
{
    kdDebug() << "MessageStore::addSentMessage" << endl;

    addMessage( MessageConstants::SMS_SUBMIT,
                receiverNumber,
                QDateTime::currentDateTime(),
                text,
                false);
}


void
MessageStore::saveMessages( const QString& messagesFile,
                            bool onlyIfModified ) const
{
    switchMessagesFile( messagesFile );
    kdDebug() << "MessageStore::saveMessages to " << messagesFile_ << endl;

    Q_ASSERT( !messagesFile_.isEmpty() );
    
    if ( !modified_ && onlyIfModified ) {
        kdDebug() << "    messages not modified" << endl;
        return;
    }

    KSaveFile xmlFile( messagesFile_ );
    
    QTextStream* out = xmlFile.textStream();

    *out << "<!DOCTYPE " << KSMS_DOCTYPE << " >" << endl;
    *out << "<messages version=\"" << KSMS_DOC_VERSION << "\">" << endl;

    for ( Message* msg = const_cast<MessageStore*>(this)->messages_.first();
          msg != 0;
          msg = const_cast<MessageStore*>(this)->messages_.next() ) {
        *out << *msg << endl;
    }
    
    *out << "</messages>" << endl;

    xmlFile.close();

    modified_ = false;
}


namespace {
QTextStream&
operator<<( QTextStream& os, const Message& msg )
{
    os << "<message "
       << "type=\""      << msg.typeString() << "\" ";
    if ( msg.timestamp().isValid() ) {
       os << "timestamp=\"" << msg.timestamp().toString(Qt::ISODate) << "\" ";
    }
    os << "sender=\""    << msg.phoneNumber() << "\">";

    const QStringList lines( QStringList::split('\r', msg.text(), true) );
    os << QStringList::split('\r', msg.text().utf8(), true).join("<br/>");
    os << "</message>";

    return os;
}
}


void
MessageStore::deleteMessage( Message* msg )
{
    kdDebug() << "MessageStore::deleteMessage: " << endl
              << msg->toString() << endl;

    MsgSender* sender = const_cast<MsgSender*>(msg->sender());
    MsgSender* receiver = const_cast<MsgSender*>(msg->receiver());
    Q_ASSERT( sender != 0 );
    Q_ASSERT( receiver != 0 );
    
    sender->removeSentMessage( msg );
    receiver->removeReceivedMessage( msg );  
    messages_.removeMessage( msg );
    
    modified_ = true;

    emit messagesChangedForSender( sender );
    emit messagesChangedForSender( receiver );
}


void
MessageStore::switchMessagesFile( const QString& messagesFile ) const
{
    if ( !messagesFile.isEmpty() ) {
        messagesFile_ = messagesFile;
    }
    if ( messagesFile_.isEmpty() ) {
        kdDebug() << "MessageStore::switchMessagesFile: no message file set!" << endl;
    }
}


MsgSender*     
MessageStore::newSender( const QString& phoneNumber )
{
    MsgSender* sender = senderFactory_->newSender( senders_, phoneNumber );

    if ( !reading_ ) {
        emit senderWasAdded( sender );
    }
    return sender;
}


void
MessageStore::emitProgress( unsigned long current,
                            unsigned long max )
{
    emit progress( current, max );
}



namespace {

MessagesParser::MessagesParser( MessageStore* store )
    : store_(store),
      state_(st_outside)
{
}


bool
MessagesParser::notationDecl( const QString& name,
                              const QString&,
                              const QString& )
{
    if ( name != KSMS_DOCTYPE ) {
        error_ = i18n( "Doctype not recognized: " ) + name;
        return false;
    }
    return true;
}


bool
MessagesParser::startElement( const QString& ,
                              const QString& ,
                              const QString& name,
                              const QXmlAttributes& attrs )
{
    if ( name == "br" ) {
        if ( state_ != st_in_message ) {
            error_ = i18n( "Line-break not inside of message." );
            return false;
        }
        msgText_ += '\r';
    }
    else if ( name == "message" ) {
        if ( state_ != st_in_messages ) {
            error_ = i18n( "Message element not inside of messages." );
            return false;
        }
        msgType_      = attrs.value( "type" ).stripWhiteSpace();
        msgTimestamp_ = attrs.value( "timestamp" ).stripWhiteSpace();
        msgSender_    = attrs.value( "sender" ).stripWhiteSpace();
        msgText_      = QString::null;
        state_ = st_in_message;
    }
    else if ( name == "messages" ) {
        if ( state_ != st_outside ) {
            error_ = i18n( "Messages element not at document top-level." );
            return false;
        }
        const QString version( attrs.value("version").stripWhiteSpace() );
        if ( version != KSMS_DOC_VERSION ) {
            error_ = i18n( "Unsupported message file version: %1. Need version %2" )
                            .arg(version)
                            .arg(KSMS_DOC_VERSION);
            return false;
        }
        state_ = st_in_messages;
    }
    return true;
}


bool
MessagesParser::endElement( const QString& ,
                            const QString& ,
                            const QString& name )
{
    static const QDateTime NULL_TIMESTAMP;

    if ( name == "message" ) {
        store_->addMessage( Message::messageTypeFromString(msgType_),
                            msgSender_,
                            msgTimestamp_.isEmpty()
                                ? NULL_TIMESTAMP
                                : QDateTime::fromString(msgTimestamp_, Qt::ISODate),
                            msgText_,
                            false );
        state_ = st_in_messages;
    }
    return true;
}


bool
MessagesParser::characters( const QString& text )
{
    if ( state_ == st_in_message ) {
        msgText_ += text;
    }
    return true;
}


QString
MessagesParser::errorString()
{
    return error_;
}

}
