/***************************************************************************
                          mobilephonebook.cpp  -  description
                             -------------------
    begin                : Sat Oct 26 2002
    copyright            : (C) 2002 by Michael Schuerig
    email                : michael@schuerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kabc/addressbook.h>
#include <kabc/stdaddressbook.h>
#include <kabc/addressee.h>
#include <kabc/phonenumber.h>
#include <kconfig.h>
#include <kemailsettings.h>
#include <kdebug.h>

#include "phoneutils.h"
#include "mobilephonebook.h"


namespace {
const QString GROUP_PHONEBOOK( "Phone Book" );
const QString RECENT_NUMBERS_COUNT( "RecentNumbersCount" );
const QString RECENT_NUMBERS( "RecentNumbers" );
}


const MobilePhoneBook::Entry MobilePhoneBook::Entry::null;



MobilePhoneBook::MobilePhoneBook( KConfig* config )
{
    if ( config ) {
        readConfig( config );
    }
}


MobilePhoneBook::~MobilePhoneBook()
{
}


const MobilePhoneBook::EntryList
MobilePhoneBook::entries() const
{
    const KABC::AddressBook* book = KABC::StdAddressBook::self();

    EntryList results;

    for ( KABC::AddressBook::ConstIterator it( book->begin() );
          it != book->end(); ++it ) {
        KABC::PhoneNumber mobile = (*it).phoneNumber(KABC::PhoneNumber::Cell);
        if (! mobile.number().isEmpty() ) {
            results << Entry( PhoneUtils::normalizePhoneNumber(mobile.number()),
                              (*it).realName(), (*it).uid() );
        }
    }
    return results;
}


const QStringList
MobilePhoneBook::recentNumbers() const
{
    return recentNumbers_;
}



void
MobilePhoneBook::rememberAsRecentNumber( const QString& number )
{
    recentNumbers_.remove( number );
    recentNumbers_.push_front( number );
    while ( recentNumbers_.size() > recentNumbersCount_ ) {
        recentNumbers_.pop_back();
    }
    emit phoneNumbersChanged();
}


const MobilePhoneBook::Entry            
MobilePhoneBook::findEntryForNumber( const QString& number )
{
    const KABC::AddressBook* book = KABC::StdAddressBook::self();
    
    for ( KABC::AddressBook::ConstIterator it( book->begin() );
          it != book->end(); ++it ) {
        KABC::PhoneNumber mobile = (*it).phoneNumber(KABC::PhoneNumber::Cell);
        const QString normNumber( PhoneUtils::normalizePhoneNumber(mobile.number()) );
        if (! normNumber.isEmpty() ) {
            if ( number == normNumber ) {
                return Entry( normNumber,
                              (*it).realName(),
                              (*it).uid() );
            }
        }
    }
    return Entry::null;
}


/**
 * There's got to be a more direct way to do this...
 * Try to find "myself" first by realname, if that fails by email
 * in the address book.
 */
const MobilePhoneBook::Entry
MobilePhoneBook::findMyself()
{
    KEMailSettings settings;
    const QString myName( settings.getSetting(KEMailSettings::RealName) );

    KABC::AddressBook* book = KABC::StdAddressBook::self();
    KABC::Addressee::List myselves = book->findByName( myName );
    if ( myselves.isEmpty() ) {
        const QString myEmail( settings.getSetting(KEMailSettings::EmailAddress) );
        myselves = book->findByEmail( myEmail );
    }

    if ( !myselves.isEmpty() ) {
        const KABC::Addressee myself( myselves.first() );
        const KABC::PhoneNumber mobile( myself.phoneNumber(KABC::PhoneNumber::Cell) );
        return MobilePhoneBook::Entry( PhoneUtils::normalizePhoneNumber(mobile.number()),
                                       myName, myself.uid() );
    }
    else {
        return MobilePhoneBook::Entry( QString::null, myName, QString::null );
    }
}


void
MobilePhoneBook::readConfig( KConfig* config )
{
    kdDebug() << "Reading phonebook config" << endl;
    Q_ASSERT( config != 0 );

    config->setGroup( GROUP_PHONEBOOK );
    recentNumbersCount_ = config->readUnsignedNumEntry( RECENT_NUMBERS_COUNT );
    recentNumbers_ = config->readListEntry( RECENT_NUMBERS );
}


void
MobilePhoneBook::saveConfig( KConfig* config ) const
{
    kdDebug() << "Saving phonebook config" << endl;
    Q_ASSERT( config != 0 );

    config->setGroup( GROUP_PHONEBOOK );
    config->writeEntry( RECENT_NUMBERS, recentNumbers_ );
}


